require 'snmp'
require 'pp'
require 'getoptlong'

opts = GetoptLong.new(
	[ "--vlan",      "-v", GetoptLong::REQUIRED_ARGUMENT ],
	[ "--community", "-c", GetoptLong::REQUIRED_ARGUMENT ],
	[ "--yaml",      "-y", GetoptLong::OPTIONAL_ARGUMENT ],
	[ "--help",      "-h", GetoptLong::NO_ARGUMENT ]
)

vlans = [ 1 ]
community = 'public'
output = '-'

opts.each do |opt, arg|
	case opt
		when '--help'
			puts "help !"; exit 1;
		when '--vlan'
			vlans = arg
		when '--community'
			community = arg
		when '--yaml'
			output = arg
	end
end

if vlans !~ /[0-9,]+/
	$stderr.puts "vlans argument should be numeric and comma separated"
	exit 1
end
vlans = vlans.split(/,/).collect { |v| v.to_i }

hosts = ARGV
if hosts.empty?
	$stderr.puts "no snmp host(s) given"
	exit 1
end

macTableOID = SNMP::ObjectId.new("1.3.6.1.2.1.17.4.3.1.1")
macToPortOID = SNMP::ObjectId.new("1.3.6.1.2.1.17.4.3.1.2")
ifNameOID = SNMP::ObjectId.new("1.3.6.1.2.1.31.1.1.1.1")
uidToIfIdOID = SNMP::ObjectId.new("1.3.6.1.2.1.17.1.4.1.2")
ifOperStatusOID = SNMP::ObjectId.new("1.3.6.1.2.1.2.2.1.8")

switchtables = {}

hosts.each do |host|
	hostname = host
	switchtype = :default
	$stderr.puts "Probe: connecting to #{host} with #{community} community"
	SNMP::Manager.open( :Host => host, :Port => 161,
		:Community => community) do |manager|
		manager.get('sysDescr.0').each_varbind do |vb|
			case vb.value.to_s
				when /ProCurve Switch/
					switchtype = :default
				when /Cisco Internetwork Operating/
					switchtype = :catalyst
				else
					$stderr.puts "Warning, this type was never tested"
			end
		end
		manager.get('sysName.0').each_varbind do |vb|
			hostname = vb.value.to_s + "[#{host}]"
		end
	end

	switchtables[hostname] = {}
	macaddresstable = {}

	vlans.each do |vlan|
		ports = {}
		interfaces = {}
		case switchtype 
			when :catalyst
				collectcommunity = "#{community}@#{vlan}"
			else
				collectcommunity = community
		end
				
		$stderr.puts "Collect: connecting to #{host} " +
				"with #{collectcommunity} community"
		SNMP::Manager.open( :Host => host, :Port => 161,
				:Community => collectcommunity) do |manager|
			manager.walk([macTableOID, macToPortOID]) do |macTable, macToPort|
				portid = macToPort.value.to_i
				ports[portid] ||= []
				mac = macTable.value
				mac = sprintf("%02x:%02x:%02x:%02x:%02x:%02x",
					mac[0],mac[1],mac[2],mac[3],mac[4],mac[5])

				ports[portid] << mac
			end
			assoc = {}
			manager.walk(uidToIfIdOID) do |uidToIfId|
				portuid = uidToIfId.oid.index(uidToIfIdOID).to_s.to_i
				portid = uidToIfId.value.to_i
				assoc[portid] = portuid
			end

			manager.walk([ifNameOID, ifOperStatusOID]) do |ifName,ifOperStatus|
				portid = ifName.oid.index(ifNameOID).to_s.to_i
				status = ifOperStatus.value.to_s == '1' ? 'up' : 'down'
				ifname = ifName.value.to_s
				ifname = ifname.to_i if ifname == ifname.to_i.to_s
				interfaces[ifname] ||= {}
				interfaces[ifname][:portid] = portid
				interfaces[ifname][:status] = status
			end

			interfaces.each do |portstring, portinfo|
				portid = portinfo[:portid]
				if assoc.has_key?(portid)
					interfaces[portstring] = { 
						:macs => ports[assoc[portid]], 
						:status => interfaces[portstring][:status]
					}
				else
					interfaces[portstring].delete(:portid)
					interfaces[portstring][:macs] = []
				end
			end	
			vlan = :default if switchtype == :default
			macaddresstable[vlan] = interfaces.dup
		end
		switchtables[hostname] = macaddresstable
		break if switchtype == :default
	end
end

if output == '-'
	puts YAML.dump(switchtables)
else
	begin
		File.open(output,'w') do |file|
			file.puts YAML.dump(switchtables)
		end
	rescue Exception => e
		$stderr.puts "Error writing file #{file}\n#{e.to_s}"
		exit 1
	end
end	
