#!/usr/bin/ruby

require 'yaml'
require 'pp'
require 'getoptlong'

Columns=[ 'Interfaces', 'MAC', 'Status', 'Comment' ]
ColumnsStripped =[ 'Inf', 'MAC Address', 'St', 'Comment' ]

opts = GetoptLong.new(
	[ "--no-down",      "-d", GetoptLong::NO_ARGUMENT ],
	[ "--no-empty",     "-e", GetoptLong::NO_ARGUMENT ],
	[ "--no-trunk",     "-t", GetoptLong::NO_ARGUMENT ],
	[ "--no-uplink",    "-u", GetoptLong::NO_ARGUMENT ],
	[ "--comment-file", "-c", GetoptLong::REQUIRED_ARGUMENT ],
	[ "--help",         "-h", GetoptLong::NO_ARGUMENT ]
)

nodown = false
noempty = false
commentfile = nil
notrunk = false
nouplink = false
opts.each do |opt, arg|
	case opt
		when '--help'
			puts "help!"; exit 1;
		when '--no-down'
			nodown = true
		when '--no-empty'
			noempty = true
		when '--comment-file'
			commentfile = arg
		when '--no-trunk'
			notrunk = true
		when '--no-uplink'
			nouplink = true
	end
end

Source=(ARGV[0] || '-')

Status = if File.exists?(Source)
	YAML::load_file(Source)
else
	YAML::load($stdin.read)
end

unless commentfile.nil?
	begin
		Comment = YAML::load_file(commentfile)
	rescue
		$stderr.puts "warning: couldn't open comment file '#{commentfile}'"
		commentfile = nil
	end
end

puts "<html>"
puts "<head><link rel='stylesheet' href='rswt.css' type='text/css' /></head>"
puts "<body>"

Status.keys.sort.each do |switch|
	switchtable = Status[switch]
	switchtable.each do |vlan,interfaces|

		$stderr.puts "Generate: #{switch}: #{vlan}"
		switchid = switch.split(/\//).first.downcase.gsub(/\s/,'') + "_#{vlan}"
		puts "<span class='switchstatus' id='#{switchid}'>"
		puts "<table class='interfaces'>"
		puts "<caption>#{switch}: #{vlan}</caption>"
		thead = ColumnsStripped.collect { |title| "<th>#{title}</th>" }
		puts "<thead><tr>#{thead}</tr></thead>"

		interfaces.keys.sort do |x,y|
				if x.class == y.class 
					if x.class == String and y.class == String
						xs = x.scan(/[A-Z][a-z]+/).first || ''
						ys = y.scan(/[A-Z][a-z]+/).first || ''
						xm = x.scan(/[0-9]+/).map { |s| s.to_i }
						ym = y.scan(/[0-9]+/).map { |s| s.to_i }
						[ xs, xm ].flatten <=> [ ys, ym ].flatten
					else
						x <=> y
					end
				else
					x.to_s <=> y.to_s
				end
			end.each do |interface|
			infos = interfaces[interface]
			macs = infos['macs'] || infos[:macs]
			if macs.nil? or macs.empty?
				next if noempty
				macs = [ '&nbsp' ]
			end
			status = infos['status'] || infos[:status]
			#comment = infos['comment'] || '&nbsp;'
			begin
				comment = Comment[switch][vlan][interface][:comment]
			rescue
				comment = '&nbsp;'
			end
			next if status == 'down' and nodown and comment !~ /trunkmember/
			next if notrunk and comment =~ /trunk/i
			next if nouplink and comment =~ /uplink/i
			next if comment =~ /^hidden$/
			trunkoruplink = true if comment =~ /(trunk|uplink)/i


			macs.each do |mac|
				notice = [ "inf#{(interface.to_s.gsub(/[^0-9]/,'').to_i % 2)}" ]
				notice << 'multmac' if macs.size > 1 and not trunkoruplink
				notice << 'down' if status == 'down'
				notice << 'up' if status == 'up'
				notice << 'pending' if status == 'up' and macs.size == 0
				notice << 'lastmac' if mac == macs[-1] or macs.empty?
				notice << 'memberdown' if comment =~ /trunkmember/ and status == 'down'
				cssclass = ''
				cssclass = "class='" + notice.join(' ') + "'" unless notice.empty?
				
				mac = '&nbsp;' if mac.empty?
				columns = [ interface, mac, status, comment ]
				line = Columns.zip(columns).collect do |content|
					"<td class='#{content[0].downcase}'>#{content[1]}</td>"
				end.to_s

				puts "<tr #{cssclass}>#{line}</tr>"
				interface = "&nbsp;"
			end
		end
		puts "</thead></table>"
		puts "</span>"
	end
end

puts "</body>"
puts "</html>"
